@@ -27,8 +27,10 @@
 #include <config.h>
 #endif
 
+#include <qintcache.h>
 #include <qobject.h>
 #include <qlist.h>
+#include <qmap.h>
 #include <kurl.h>
 
 #include "catalogsettings.h"
@@ -61,13 +63,24 @@ struct PoInfo
    int fuzzy;
    int untranslated;
 
+   QString project;
+   QString creation;
+   QString revision;
    QString lastTranslator;
    QString languageTeam;
-   QString revision;
-   QString project;
+   QString mimeVersion;
+   QString contentType;
+   QString encoding;
+   QString others;
+
+   QString headerComment;
 };
 
-
+struct DiffEntry
+{
+    QString msgid;
+    QString msgstr;
+};
 
 /**
 * This class represents a catalog, saved in a po-file.
@@ -83,6 +96,7 @@ class Catalog : public QObject
 
 public:
    enum IOStatus{OK,OS_ERROR,NO_PERMISSIONS,RECOVERED_PARSE_ERROR,PARSE_ERROR,NO_FILE};
+   enum DiffResult{DiffOk, DiffNotFound, DiffNeedList};
 
    static IOStatus info(const QString url,PoInfo& info);
    static PoInfo headerInfo(const CatalogItem);
@@ -105,6 +119,12 @@ public:
    QString comment(uint index) const;
    CatalogItem header() const;
 
+   /**
+    * @return The index of the item, that has the msgid id.
+    * If no item is found, -1 is returned.
+    */
+   int indexForMsgid(const QString& id) const;
+   
    /** @return a list of tags in entry #index */
    QStringList tagList(uint index);
 
@@ -146,6 +166,7 @@ public:
    KURL currentURL() const;
    QString packageName() const;
    QString packageDir() const;
+   QString encoding() const;
 
    /**
    * opens file url by using KDE's network downlad and calls
@@ -207,6 +228,12 @@ public:
    */
    bool checkForContext(bool clearErrors=true);
 
+   /**
+    * checks if a string which contains both singular and plural form
+    * is translated correctly
+    */
+   bool checkSingularPlural(bool clearErrors=true);
+
    /** closes the file and deletes all entries */
    void clear();
 
@@ -228,6 +255,7 @@ public:
    bool isFuzzy(uint index) const;
    bool isUntranslated(uint index) const;
    bool hasError(uint index) const;
+   bool isPluralForm(uint index) const;
 
    /**
    * returns index of next fuzzy entry behind startIndex
@@ -323,6 +351,33 @@ public:
 
    void removeFromErrorList(uint index);
 
+   /**
+    * tries to find a corresponding entry for entry entry 
+    * from the list of old messages and calculates the diff for it
+    */
+   DiffResult diff(uint entry, QString* result);
+
+   /**
+    * sets a list of entries to generate a diff from
+    */
+   void setDiffList( const QValueList<DiffEntry>& );
+
+   /**
+    * @return the contents of this catalog as list for diffs
+    */
+   QValueList<DiffEntry> asDiffList();
+   
+   /**
+    * @return how many plural forms are used in language lang.
+    * If nothing is found -1 is returned.
+    */
+   static int getNumberOfPluralForms(const QString& lang);
+
+   /**
+    * @return true, if this item has gettext plural forms
+    */
+   bool hasPluralForms() const;
+
 public slots:
    void setSettings(SaveSettings settings);
    void setSettings(IdentitySettings settings);
@@ -370,6 +425,13 @@ private:
 
    void clearErrorList();
 
+   /**
+    * Tries to find out how many plural forms are used in this language and
+    * sets numberOfPluralForms accordingly. If nothing is found,
+    * numberOfPLuralForms is set to -1.
+    */
+   void getNumberOfPluralForms();
+
 private:
    /** url of the po-file, that belongs to this catalog */
    KURL _url;
@@ -397,6 +459,12 @@ private:
 
    QTextCodec *fileCodec;
 
+   QStringList msgidDiffList;
+   QMap< QString, QStringList > msgstr2MsgidDiffList;
+   QIntCache<QString> diffCache;
+   
+   int numberOfPluralForms;
+
 signals:
    void signalError(QString);
    void signalResetProgressBar(QString,int);
