@@ -128,7 +128,7 @@ if(!error)
   author=catalog->headerInfo(catalog->header()).lastTranslator;
 else author=QString("unknown");
  
-int catnum=dm->catalogRef(location,author);
+int catnum=dm->catalogRef(location,author,fileName);
 
 uint i,tot;
 tot=catalog->numberOfEntries();
@@ -180,3 +180,4 @@ delete catalog;
 return true;
 
 }
+#include "dbscan.moc"
